Index: apps/app_enumlookup.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_enumlookup.c,v
retrieving revision 1.14
diff -u -r1.14 app_enumlookup.c
--- apps/app_enumlookup.c	21 Apr 2005 06:02:43 -0000	1.14
+++ apps/app_enumlookup.c	4 May 2005 23:50:57 -0000
@@ -1,14 +1,17 @@
 /*
  * Asterisk -- A telephony toolkit for Linux.
  *
- * Time of day - Report the time of day
+ * ENUM Lookup - Old Fashioned and Next Generation
  * 
  * Copyright (C) 1999 - 2005, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
+ * EnumLookupNG added by Edwin Groothuis <edwin@mavetju.org>
+ *
  * This program is free software, distributed under the terms of
  * the GNU General Public License
+ *
  */
 
 #include "asterisk/lock.h"
@@ -27,13 +30,26 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-static char *tdesc = "ENUM Lookup";
+static char *tdesc = "ENUM Lookup apps";
+
+static char *app_enumlookupng = "EnumLookupNG";
 
-static char *app = "EnumLookup";
+static char *synopsis_enumlookupng = "Lookup number in ENUM - Next Generation Implementation";
 
-static char *synopsis = "Lookup number in ENUM";
+static char *descrip_enumlookupng = 
+"EnumLookup(exten):  Looks up an extension via ENUM, sets the variables 'ENUMDEST', 'ENUMTECH' and 'ENUMDETAIL' (for VOIP).\n"
+"For VoIP URIs this variable will look like 'TECHNOLOGY/URI' with the appropriate technology.\n"
+"Returns -1 on hangup, or 0 on completion regardless of whether the lookup was successful. \n"
+"If the technology is TEL, ENUMTECH is set to TEL and ENUMDEST has the number to dial.\n"
+"If the technology is MAILTO, ENUMTECH is set to MAILTO and ENUMDEST has the email address.\n"
+"If the technology is SIP, H323, IAX or IAX2, ENUMTECH is set to VOIP, ENUMDETAIL is set to the technology and ENUMDEST has first parameter for the Dial command.\n"
+"If technology is unknown or something else, ENUMTECH is set to fail and ENUMDETAIL contains (hopefully) useful information about what happened.\n";
 
-static char *descrip = 
+static char *app_enumlookup = "EnumLookup";
+
+static char *synopsis_enumlookup = "Lookup number in ENUM";
+
+static char *descrip_enumlookup = 
 "  EnumLookup(exten):  Looks up an extension via ENUM and sets\n"
 "the variable 'ENUM'. For VoIP URIs this variable will \n"
 "look like 'TECHNOLOGY/URI' with the appropriate technology.\n"
@@ -45,6 +61,7 @@
 "If the lookup was *not* successful and there exists a priority n + 101,\n"
 "then that priority will be taken next.\n" ;
 
+
 #define ENUM_CONFIG "enum.conf"
 
 static char h323driver[80] = "";
@@ -80,6 +97,7 @@
 				c += 4;
 			snprintf(tmp, sizeof(tmp), "SIP/%s", c);
 			pbx_builtin_setvar_helper(chan, "ENUM", tmp);
+			ast_log(LOG_NOTICE, "sip: ENUM set to \"%s\"\n", tmp);
 		} else if (!strcasecmp(tech, "h323")) {
 			c = dest;
 			if (!strncmp(c, "h323:", 5))
@@ -140,6 +158,107 @@
 	return res;
 }
 
+/*--- enumlookupng_exec: */
+static int enumlookupng_exec(struct ast_channel *chan, void *data)
+{
+	int res=0;
+	char tech[80];
+	char dest[80];
+	char tmp[256];
+	char *c,*t;
+	struct localuser *u;
+
+	if (!data || ast_strlen_zero(data)) {
+		ast_log(LOG_WARNING, "EnumLookupNG: missing parameters\n");
+		return 1;
+	}
+
+	LOCAL_USER_ADD(u);
+	if (!res) {
+		res = ast_get_enum(chan, data, dest, sizeof(dest), tech, sizeof(tech));
+		printf("ENUM got '%d'\n", res);
+	}
+	LOCAL_USER_REMOVE(u);
+
+	/* Parse it out */
+	if (res > 0) {
+		if (!strcasecmp(tech, "SIP")) {
+			c = dest;
+			if (!strncmp(c, "sip:", 4))
+				c += 4;
+			snprintf(tmp, sizeof(tmp), "SIP/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUMDEST", tmp);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "VOIP");
+			pbx_builtin_setvar_helper(chan, "ENUMDETAIL", "SIP");
+		} else if (!strcasecmp(tech, "h323")) {
+			c = dest;
+			if (!strncmp(c, "h323:", 5))
+				c += 5;
+			snprintf(tmp, sizeof(tmp), "%s/%s", h323driver, c);
+/* do a s!;.*!! on the H323 URI */
+			t = strchr(c,';');
+			if (t) 
+				*t = 0;
+			pbx_builtin_setvar_helper(chan, "ENUMDEST", tmp);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "VOIP");
+			pbx_builtin_setvar_helper(chan, "ENUMDETAIL", "H323");
+		} else if (!strcasecmp(tech, "iax")) {
+			c = dest;
+			if (!strncmp(c, "iax:", 4))
+				c += 4;
+			snprintf(tmp, sizeof(tmp), "IAX/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUMDEST", tmp);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "VOIP");
+			pbx_builtin_setvar_helper(chan, "ENUMDETAIL", "IAX");
+		} else if (!strcasecmp(tech, "iax2")) {
+			c = dest;
+			if (!strncmp(c, "iax2:", 5))
+				c += 5;
+			snprintf(tmp, sizeof(tmp), "IAX2/%s", c);
+			pbx_builtin_setvar_helper(chan, "ENUMDEST", tmp);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "VOIP");
+			pbx_builtin_setvar_helper(chan, "ENUMDETAIL", "IAX2");
+		} else if (!strcasecmp(tech, "mailto")) {
+			c = dest;
+			if (!strncmp(c, "mailto:", 7))
+				c += 7;
+			pbx_builtin_setvar_helper(chan, "ENUMDEST", c);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "MAILTO");
+		} else if (!strcasecmp(tech, "tel")) {
+			c = dest;
+			if (!strncmp(c, "tel:", 4))
+				c += 4;
+
+			if (c[0] != '+') {
+				ast_log(LOG_NOTICE, "tel: uri must start with a \"+\" (got '%s')\n", c);
+				res = 0;
+			} else {
+/* now copy over the number, skipping all non-digits and stop at ; or NULL */
+				t = tmp;	
+				while( *c && (*c != ';') && (t - tmp < (sizeof(tmp) - 1))) {
+					if (isdigit(*c))
+						*t++ = *c;
+					c++;
+				}
+				*t = 0;
+				pbx_builtin_setvar_helper(chan, "ENUMDEST", tmp);
+				pbx_builtin_setvar_helper(chan, "ENUMTECH", "TEL");
+			}
+		} else if (!ast_strlen_zero(tech)) {
+			ast_log(LOG_NOTICE, "Don't know how to handle technology '%s'\n", tech);
+			pbx_builtin_setvar_helper(chan, "ENUMTECH", "FAIL");
+			pbx_builtin_setvar_helper(chan, "ENUMDETAIL", tech);
+			res = 0;
+		}
+	}
+	if (!res) {
+		pbx_builtin_setvar_helper(chan, "ENUMTECH", "FAIL");
+		pbx_builtin_setvar_helper(chan, "ENUMDETAIL", "unknown");
+	} else if (res > 0)
+		res = 0;
+	return res;
+}
+
 static int load_config(void)
 {
 	struct ast_config *cfg;
@@ -162,14 +281,22 @@
 
 int unload_module(void)
 {
+	int res;
+
 	STANDARD_HANGUP_LOCALUSERS;
-	return ast_unregister_application(app);
+	res=ast_unregister_application(app_enumlookup);
+	res|=ast_unregister_application(app_enumlookupng);
+	return res;
 }
 
 int load_module(void)
 {
 	int res;
-	res = ast_register_application(app, enumlookup_exec, synopsis, descrip);
+
+	/* Register EnumLookup */
+	res = ast_register_application(app_enumlookup,enumlookup_exec,synopsis_enumlookup,descrip_enumlookup);
+	/* Register EnumLookupNG */
+	res|=ast_register_application(app_enumlookupng,enumlookupng_exec,synopsis_enumlookupng,descrip_enumlookupng);
 	if (res)
 		return(res);
 	if ((res=load_config())) {
